@@ -3,262 +3,64 @@
 # for bug reports, help or feature requests.
 
 
-# function which reformats a coefficient with two decimal places
-coeftostring <- function(x, lead.zero=FALSE, digits=2) {
-  if (is.na(x)) {
-    return("")
+# display version number and date when the package is loaded
+.onAttach <- function(libname, pkgname) {
+  vers <- help(package="texreg")$info[[1]][2]
+  dat <- help(package="texreg")$info[[1]][3]
+  packageStartupMessage(paste(vers, "\n", dat, sep=""))
+}
+
+
+# function which conflates a matrix with duplicate row names
+rearrangeMatrix <- function(m) {
+  
+  # The following code block rearranges a matrix with duplicate row names such 
+  # that these rows are conflated where possible. First, an empty matrix q with
+  # the same width is created. The rows will be copied iteratively into this 
+  # matrix. Second, we go through the unique row names, and for each row name 
+  # we create a small virtual matrix in which the values will be nicely 
+  # rearranged. After rearranging the values, this small matrix is rbinded to 
+  # the q matrix. Rearranging works in the following way (the inner loop): for 
+  # every column, we create a vector of all values corresponding to the specific
+  # row name (as specified by the outer loop). We retain only non-NA values 
+  # because irrelevant information should be removed from the coefficients 
+  # table. Then we put the first non-NA value in the first vertical slot of the 
+  # virtual matrix, the second non-NA value of the same row name in the second 
+  # slot, etc., and we create additional rows in the virtual matrix as needed.
+  # By doing this, we ensure that no space in the matrix is wasted with NA 
+  # values. When going to the next column, we place the non-NA values in the 
+  # correct slot again, and we only create new rows if needed. The virtual rows 
+  # are finally rbinded to the large replacement matrix q.
+  
+  unique.names <- unique(rownames(m))              #unique row names in m
+  num.unique <- length(unique.names)               #count these unique names
+  orig.width <- length(m[1,])                      #number of columns in m
+  q <- matrix(nrow=0, ncol=orig.width)             #new matrix with same width
+  for (i in 1:num.unique) {                        #go through unique row names
+    rows <- matrix(NA, nrow=0, ncol=orig.width)    #create matrix where re-
+                                                   #arranged rows will be stored
+    for (j in 1:orig.width) {                      #go through columns in m
+      current.name <- unique.names[i]              #save row name
+      nonNa <- m[rownames(m)==current.name,j]      #create a vector of values
+                                                   #with same rowname in the col
+      nonNa <- nonNa[!is.na(nonNa)]                #retain only non-NA values
+      for (k in 1:length(nonNa)) {                 #go through non-NA values
+        if (k > dim(rows)[1]) {                    #add an NA-only row in which
+          rows <- rbind(rows, rep(NA, orig.width)) #the values are stored
+          rownames(rows)[k] <- unique.names[i]     #also add the row name
+        }
+        rows[k,j] <- nonNa[k]                      #actually store the value
+      }
+    }
+    q <- rbind(q, rows)                            #add the new row(s) to q
   }
-  if (digits < 0) {
-    stop("The number of digits must be 0 or higher.")
-  }
-  y <- as.character(round(x, digits))
-  
-  actual.digits <- attributes(regexpr("\\.[0-9]+$", y))$match.length - 1
-  if (grepl("\\.", y) == FALSE) { #no decimal places present
-    zeros <- paste(rep("0", digits), collapse="")
-    y <- paste(y, ".", zeros, sep="")
-  } else if (grepl("\\.[0-9]$", y) == TRUE && digits > 1) { #only one decimal p.
-    zeros <- paste(rep("0", digits-1), collapse="")
-    y <- paste(y, zeros, sep="")
-  } else if (actual.digits < digits) { #more desired digits than present
-    fill <- digits - actual.digits
-    zeros <- paste(rep("0", fill), collapse="")
-    y <- paste(y, zeros, sep="")
-  }
-  if (grepl("\\.$", y) == TRUE) {
-    y <- gsub("\\.$", "", y)
-  }
-  if (lead.zero==FALSE && (grepl("^0",y) == TRUE || grepl("^-0",y) == TRUE)) {
-    y <- gsub("0\\.", "\\.", y)
-  }
-  return(y)
+  return(q)
 }
 
 
-# generic extract function
-setGeneric("extract", function(model, ...) standardGeneric("extract"), 
-    package="texreg")
-
-
-# extension for clogit objects (survival package); submitted by Sebastian Daza
-extract.clogit <- function(model) {
-  tab <- summary(model)$coef[,c(-2,-4)]
-  aic <- extractAIC(model)[2]
-  event <- model$nevent
-  n <- model$n
-  mis <- length(model$na.action)
-  gof <- matrix(c(aic, event, n, mis), ncol=1)
-  row.names(gof) <- c("AIC", "Events", "Num. obs.", "Missings")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("clogit", "survival"), 
-    definition = extract.clogit)
-
-
-# extension for ergm objects
-extract.ergm <- function(model) {
-  tab <- summary(model)$coefs[,-3] #extract coefficient table
-  
-  lik <- model$mle.lik[1] #extract log likelihood
-  aic <- summary(model)$aic #extract AIC
-  bic <- summary(model)$bic #extract BIC
-  gof <- matrix(c(aic, bic, lik), ncol=1)
-  row.names(gof) <- c("AIC", "BIC", "Log Likelihood")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("ergm", "ergm"), 
-    definition = extract.ergm)
-
-
-# extension for glm objects
-extract.glm <- function(model) {
-  tab <- summary(model)$coef[,-3] #extract coefficient table
-  
-  aic <- summary(model)$aic #extract AIC
-  n <- nobs(model) #extract number of observations
-  
-  gof <- matrix(c(aic, n), ncol=1)
-  row.names(gof) <- c("AIC", "Num. obs.")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("glm", "stats"), 
-    definition = extract.glm)
-
-
-# extension for gls objects
-extract.gls <- function(model) {
-  tab <- summary(model)$tTable[,-3] #extract coefficient table
-  
-  lik <- summary(model)$logLik #extract log likelihood
-  aic <- summary(model)$AIC #extract AIC
-  bic <- summary(model)$BIC #extract BIC
-  n <- nobs(model) #extract number of observations
-  gof <- matrix(c(aic, bic, lik, n), ncol=1)
-  row.names(gof) <- c("AIC", "BIC", "Log Likelihood", "Num. obs.")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("gls", "nlme"), 
-    definition = extract.gls)
-
-
-# extension for lm objects
-extract.lm <- function(model) {
-  tab <- summary(model)$coef[,-3] #extract coefficient table
-  
-  rs <- summary(model)$r.squared #extract R-squared
-  adj <- summary(model)$adj.r.squared #extract adjusted R-squared
-  n <- nobs(model) #extract number of observations
-  
-  gof <- matrix(c(rs, adj, n), ncol=1)
-  row.names(gof) <- c("R$^2$", "Adj. R$^2$", "Num. obs.")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("lm", "stats"), 
-    definition = extract.lm)
-
-
-# extension for lme objects
-extract.lme <- function(model) {
-  tab <- summary(model)$tTable[,-3:-4] #extract coefficient table
-  
-  lik <- summary(model)$logLik #extract log likelihood
-  aic <- summary(model)$AIC #extract AIC
-  bic <- summary(model)$BIC #extract BIC
-  n <- nobs(model) #extract number of observations
-  gof <- matrix(c(aic, bic, lik, n), ncol=1)
-  row.names(gof) <- c("AIC", "BIC", "Log Likelihood", "Num. obs.")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("lme", "nlme"), 
-    definition = extract.lme)
-
-
-# extension for lrm objects (Design or rms package); submitted by Fabrice Le Lec
-extract.lrm <- function(model) {
-  tab <- model$coef #extract coefficient table
-  
-  attributes(model$coef)$names <- lapply(attributes(model$coef)$names, 
-    function(x) gsub(">=", " $\\\\geq$ ", x))
-  
-  tab <- cbind(COEFEST = model$coef, SE = sqrt(diag(model$var)), 
-      PVALUES = pnorm(abs(model$coef/sqrt(diag(model$var))), 
-      lower.tail = FALSE)*2)
-  
-  pseudors <- model$stats[10] #extract pseudo R-squared
-  LR <- model$stats[3] #extract LR
-  n <- model$stats[1] #extract number of observations
-  gof <- matrix(c(pseudors, LR, n), ncol=1)
-  row.names(gof) <- c("Pseudo R$^2$", "L.R.", "Num. obs.")
-  
-  table.content <- list(tab, gof) #put coefficients and gofs in a list
-  return(table.content) #return the list object
-}
-
-setMethod("extract", signature=className("lrm", "rms"), 
-    definition = extract.lrm)
-setMethod("extract", signature=className("lrm", "Design"), 
-    definition = extract.lrm)
-
-
-# extension for plm objects (from the plm package); submitted by Lena Koerber
-extract.plm <- function(model) {
-  tab <- summary(model)$coef[,-3]
-  
-  rs <- summary(model)$r.squared
-  n <- length(summary(model)$resid)
-  gof <- matrix(c(rs, n), ncol=1)
-  row.names(gof) <- c("R$^2$", "Adj. R$^2$", "Num. obs.")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("plm", "plm"), 
-    definition = extract.plm)
-
-
-# extension for pmg objects (from the plm package); submitted by Lena Koerber
-extract.pmg <- function(model) {
-  co <- data.matrix(summary(model)$coef)
-  se <- (diag(summary(model)$vcov))^(1/2) #standard errors
-  t <- co / se #t-statistics
-  n <- length(summary(model)$resid) #number of observations
-  d <- n - length(co) #degrees of freedom
-  pval <- 2 * pt(-abs(t), df=d)
-  tab <- cbind(co, se, pval) #coefficient table
-  
-  gof <- matrix(n, ncol=1)
-  row.names(gof) <- c("Num. obs.")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("pmg", "plm"), 
-    definition = extract.pmg)
-
-
-# extension for rq objects (quantreg package); submitted by Lena Koerber
-extract.rq <- function(model) {
-  tab <- summary(model, cov=TRUE)$coef[,-3]
-  n <- length(summary(model)$resid)
-  tau<-summary(model)$tau
-  gof <- matrix(c(n, tau), ncol=1)
-  row.names(gof) <- c("Num. obs.", "Percentile")
-  
-  table.content <- list(tab, gof)
-  return(table.content)
-}
-
-setMethod("extract", signature=className("rq", "quantreg"), 
-    definition = extract.rq)
-
-
-# extension for systemfit objects; submitted by Johannes Kutsam
-extract.systemfit <- function(model) {
-  equationList <- list()
-  for(eq in model$eq){  #go through estimated equations
-    sum <- summary(eq)  #extract model summary
-    tab <- coef(sum)[,-3]  #coefficients table for the equation
-    
-    rsquared <- sum$r.squared  #extract r-squared
-    radj <- sum$adj.r.squared  #extract adjusted r-squared
-    n <- nobs(model)  #extract number of observations
-    
-    gof <- matrix(c(rsquared, radj, n), ncol=1)  #GOF matrix
-    row.names(gof) <- c("R$^2$", "Adj. R$^2$", "Num. obs.")  #set GOF row names
-
-    table.content <- list(tab, gof)
-    equationList[[eq$eqnNo]] <- table.content
-  }
-  return(equationList)  #returns a list of table.content lists
-}
-
-setMethod("extract", signature=className("systemfit", "systemfit"), 
-    definition = extract.systemfit)
-
-
 # texreg function
 texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE, 
-    table=TRUE, sideways=FALSE, float.pos="", strong.signif=FALSE, 
+    table=TRUE, sideways=FALSE, float.pos="", stars=TRUE, strong.signif=FALSE, 
     symbol="\\cdot", use.packages=TRUE, caption="Statistical models", 
     label="table:coefficients", dcolumn=TRUE, booktabs=TRUE, scriptsize=FALSE, 
     custom.names=NA, model.names=NA, digits=2, ...) {
@@ -266,7 +68,7 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
   string <- ""
   
   # if a single model is handed over, put model inside a list
-  if (class(l) != "list") {
+  if (!"list" %in% class(l)) {
     l <- list(l)
   }
 
@@ -274,35 +76,54 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
   models <- NULL
   for (i in 1:length(l)) {
     model <- extract(l[[i]], ...)
-    if (class(model[[1]]) == "list") {  #nested list of models (e.g. systemfit)
+    if (class(model) == "list") {       #nested list of models (e.g. systemfit)
       models <- append(models, model)
-    } else {  #normal case; one model
+    } else {                            #normal case; one model
       models <- append(models, list(model))
     }
   }
   
   # extract names of the goodness-of-fit statistics
-  gof.names <- character()
+  gof.names <- character()  #names of all models in one vector
   for (i in 1:length(models)) {
-    for (j in 1:length(models[[i]][[2]])) {
-      if (!row.names(models[[i]][[2]])[j] %in% gof.names) {
-        gof.names <- append(gof.names, row.names(models[[i]][[2]])[j])
+    gn <- models[[i]]@gof.names
+    for (j in 1:length(gn)) {
+      if (!gn[j] %in% gof.names) {
+        gof.names <- append(gof.names, gn[j])
       }
     }
   }
   
-  # aggregate goodness-of-fit statistics in a matrix and create list of coefs
+  # aggregate GOF statistics in a matrix and create list of coef blocks
   coefs <- list()
   gofs <- matrix(nrow=length(gof.names), ncol=length(models))
+  decimal.matrix <- matrix(nrow=length(gof.names), ncol=length(models))
   row.names(gofs) <- gof.names
   for (i in 1:length(models)) {
-    coefs <- append(coefs, models[[i]][1])
-    for (j in 1:length(models[[i]][[2]])) {
-      rn <- row.names(models[[i]][[2]])[j]
-      val <- models[[i]][[2]][[j]]
+    cf <- models[[i]]@coef
+    se <- models[[i]]@se
+    pv <- models[[i]]@pvalues
+    if (length(pv) > 0) {
+      coef <- cbind(cf, se, pv)
+    } else {
+      coef <- cbind(cf, se, rep(0.99, length(cf)))
+    }
+    rownames(coef) <- models[[i]]@coef.names
+    coefs[[i]] <- coef
+    for (j in 1:length(models[[i]]@gof)) {
+      rn <- models[[i]]@gof.names[j]
+      val <- models[[i]]@gof[j]
       col <- i
+      if (is.na(models[[i]]@gof.decimal[j])) {
+        dec <- digits
+      } else if (models[[i]]@gof.decimal[j] == FALSE) {
+        dec <- 0
+      } else {
+        dec <- digits
+      }
       row <- which(row.names(gofs) == rn)
       gofs[row,col] <- val
+      decimal.matrix[row,col] <- dec
     }
   }
   
@@ -359,31 +180,7 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
     rownames(m) <- custom.names
   }
   
-  # check if the custom name procedure caused duplicates and merge them
-  for (i in 1:length(rownames(m))) {  #go through rows
-    for (j in 1:length(rownames(m))) {  #go through rows (to find duplicates)
-      if (i != j & rownames(m)[i] == rownames(m)[j]) {  #found a duplicate name
-        identical <- logical(length(m[i,]))
-        for (k in 1:length(m[i,])) {  #go through columns
-          if ( (is.na(m[i,k]) & !is.na(m[j,k])) | 
-              (!is.na(m[i,k]) & is.na(m[j,k])) | 
-              (is.na(m[i,k]) & is.na(m[j,k])) ) {
-            identical[k] <- TRUE  #set TRUE if they are complementary
-          }
-        }
-        if (length(identical[identical==FALSE]) == 0) {  #if complementary...
-          for (k in 1:ncol(m)) {  #go through the columns again
-            if (is.na(m[i,k])) {
-              m[i,k] <- m[j,k]  #merge them
-            } else if (is.na(m[j,k])) {
-              m[j,k] <- m[i,k]  #merge them
-            }
-          }
-        }
-      }
-    }
-  }
-  m <- m[duplicated(m) == FALSE,]  #remove duplicate rows
+  m <- rearrangeMatrix(m)
   m <- as.data.frame(m)
   
   # what is the optimal length of the labels?
@@ -535,11 +332,11 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
         if (is.na(m[i,j])) {
           output.matrix[i,k] <- ""
         } else if (m[i,j] == -Inf) {
-          output.matrix[i,k] <- "-Inf (NA)"
+          output.matrix[i,k] <- "\\multicolumn{1}{c}{$-$Inf}"
         } else {
           std <- paste(" \\; (", coeftostring(m[i,j+1], leading.zero, 
               digits=digits), ")", sep="")
-          if (strong.signif == TRUE) {
+          if (strong.signif == TRUE && stars==TRUE) {
             if (m[i,j+2] <= 0.001) {
               p <- "^{***}"
             } else if (m[i,j+2] <= 0.01) {
@@ -551,7 +348,7 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
             } else {
               p <- ""
             }
-          } else {
+          } else if (stars==TRUE) {
             if (m[i,j+2] <= 0.01) {
               p <- "^{***}"
             } else if (m[i,j+2] <= 0.05) {
@@ -561,6 +358,8 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
             } else {
               p <- ""
             }
+          } else {
+              p <- ""
           }
           if (dcolumn == TRUE) {
             dollar <- ""
@@ -590,14 +389,14 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
       j <- 1
       k <- 2
       while (j <= length(m)) {
-        if (is.na(m[i,j])) {
+        if (is.na(m[i,j]) || is.nan(m[i,j])) {
           output.matrix[(i*2)-1,k] <- "" #upper coefficient row
           output.matrix[(i*2),k] <- "" #lower std row
         } else if (m[i,j] == -Inf) {
-          output.matrix[(i*2)-1,k] <- "-Inf" #upper coefficient row
-          output.matrix[(i*2),k] <- "(NA)" #lower std row
+          output.matrix[(i*2)-1,k] <- "\\multicolumn{1}{c}{$-$Inf}" #upper row
+          output.matrix[(i*2),k] <- "" #lower std row
         } else {
-          if (strong.signif == TRUE) {
+          if (strong.signif == TRUE && stars==TRUE) {
             if (m[i,j+2] <= 0.001) {
               p <- "^{***}"
             } else if (m[i,j+2] <= 0.01) {
@@ -609,7 +408,7 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
             } else {
               p <- ""
             }
-          } else {
+          } else if (stars==TRUE) {
             if (m[i,j+2] <= 0.01) {
               p <- "^{***}"
             } else if (m[i,j+2] <= 0.05) {
@@ -619,6 +418,8 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
             } else {
               p <- ""
             }
+          } else {
+              p <- ""
           }
           if (dcolumn == TRUE) {
             dollar <- ""
@@ -646,26 +447,7 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
   for (i in 1:length(gofs[,1])) {
     gof.matrix[i,1] <- rownames(gofs)[i]
     for (j in 1:length(gofs[1,])) {
-      strg <- coeftostring(gofs[i,j], leading.zero, digits=digits)
-      rn <- rownames(gofs)[i]
-      if (rn == "Num. obs." | rn == "n" | rn == "N" | rn == "N obs" | 
-          rn == "N obs." | rn == "nobs" | rn == "n obs" | rn == "n obs." | 
-          rn == "n.obs." | rn == "N.obs." | rn == "N. obs" | 
-          rn == "Num observations" | rn == "Number of observations" | 
-          rn == "Num obs" | rn == "num obs" | rn == "Num. observations" | 
-          rn == "Num Observations" | rn == "Num. Observations" | 
-          rn == "Num. Obs." | rn == "Num.Obs." | rn == "Number obs." | 
-          rn == "Number Obs." | rn == "Number obs" | rn == "Number Obs" | 
-          rn == "Number of Obs." | rn == "Number of obs." | 
-          rn == "Number of obs" | rn == "Number of Obs" | rn == "Obs" | 
-          rn == "obs" | rn == "Obs." | rn == "obs." | rn == "Events" | 
-          rn == "# Events" | rn == "# events" | rn == "events" | 
-          rn == "Missings" | rn == "Missing" | rn == "# Missing") {
-        strg <- strsplit(strg, "\\.")[[1]][1]
-        if (is.na(strg) == TRUE) {
-          strg <- ""
-        }
-      }
+      strg <- coeftostring(gofs[i,j], leading.zero, digits=decimal.matrix[i,j])
       gof.matrix[i,j+1] <- paste(dollar, strg, dollar, sep="")
     }
   }
@@ -693,7 +475,6 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
     }
   }
   
-  
   # write coefficients to string object
   for (i in 1:(length(output.matrix[,1])-length(gof.names))) {
     for (j in 1:length(output.matrix[1,])) {
@@ -732,15 +513,15 @@ texreg <- function(l, single.row=FALSE, no.margin=TRUE, leading.zero=TRUE,
   }
   string <- paste(string, "\\vspace{-2mm}\\\\\n", sep="")
   
-  if (strong.signif == TRUE) {
+  if (strong.signif == TRUE && stars==TRUE) {
     string <- paste(string, "\\multicolumn{", length(models)+1, 
-        "}{l}{\\textsuperscript{***}$p<0.001$, ", 
-        "\\textsuperscript{**}$p<0.01$, \\textsuperscript{*}$p<0.05$, ", 
-        "\\textsuperscript{$", symbol, "$}$p<0.1$}\n", sep="")
-  } else {
+        "}{l}{\\textsuperscript{***}$p<0.001$, \n", 
+        "  \\textsuperscript{**}$p<0.01$, \n  \\textsuperscript{*}$p<0.05$, \n",
+        "  \\textsuperscript{$", symbol, "$}$p<0.1$}\n", sep="")
+  } else if (stars==TRUE) {
     string <- paste(string, "\\multicolumn{", length(models)+1, 
-        "}{l}{\\textsuperscript{***}$p<0.01$, ", 
-        "\\textsuperscript{**}$p<0.05$, \\textsuperscript{*}$p<0.1$}\n", 
+        "}{l}{\\textsuperscript{***}$p<0.01$, \n", 
+        "  \\textsuperscript{**}$p<0.05$, \n  \\textsuperscript{*}$p<0.1$}\n", 
         sep="")
   }
   
